@@ -1,3 +1,5 @@
+use std::{collections::HashSet, marker::PhantomData};
+
 use poem::{
     endpoint::{make_sync, BoxEndpoint},
     middleware::CookieJarManager,
@@ -7,8 +9,8 @@ use poem::{
 
 use crate::{
     base::UrlQuery,
-    registry::{Document, MetaInfo, MetaLicense, MetaServer, Registry},
-    OpenApi,
+    registry::{Document, MetaExternalDocument, MetaInfo, MetaLicense, MetaServer, Registry},
+    OpenApi, Webhook,
 };
 
 /// An object representing a Server.
@@ -18,24 +20,12 @@ pub struct ServerObject {
     description: Option<String>,
 }
 
-impl From<String> for ServerObject {
-    fn from(url: String) -> Self {
+impl<T: Into<String>> From<T> for ServerObject {
+    fn from(url: T) -> Self {
         Self::new(url)
     }
 }
 
-impl From<&str> for ServerObject {
-    fn from(url: &str) -> Self {
-        Self::new(url.to_string())
-    }
-}
-
-impl From<&str> for LicenseObject {
-    fn from(name: &str) -> Self {
-        Self::new(name.to_string())
-    }
-}
-
 impl ServerObject {
     /// Create a server object by url.
     pub fn new(url: impl Into<String>) -> ServerObject {
@@ -45,7 +35,7 @@ impl ServerObject {
         }
     }
 
-    /// Specifies an string describing the host designated by the URL.
+    /// Sets an string describing the host designated by the URL.
     pub fn description(self, description: impl Into<String>) -> Self {
         Self {
             description: Some(description.into()),
@@ -61,9 +51,9 @@ pub struct LicenseObject {
     url: Option<String>,
 }
 
-impl From<String> for LicenseObject {
-    fn from(name: String) -> Self {
-        Self::new(name)
+impl<T: Into<String>> From<T> for LicenseObject {
+    fn from(url: T) -> Self {
+        Self::new(url)
     }
 }
 
@@ -77,7 +67,7 @@ impl LicenseObject {
         }
     }
 
-    /// Specifies an [`SPDX`](https://spdx.org/spdx-specification-21-web-version#h.jxpfx0ykyb60) license expression for the API.
+    /// Sets an [`SPDX`](https://spdx.org/spdx-specification-21-web-version#h.jxpfx0ykyb60) license expression for the API.
     pub fn identifier(self, identifier: impl Into<String>) -> Self {
         Self {
             identifier: Some(identifier.into()),
@@ -85,7 +75,7 @@ impl LicenseObject {
         }
     }
 
-    /// Specifies a URL to the license used for the API.
+    /// Sets a URL to the license used for the API.
     pub fn url(self, url: impl Into<String>) -> Self {
         Self {
             url: Some(url.into()),
@@ -94,20 +84,54 @@ impl LicenseObject {
     }
 }
 
+/// An object representing a external document.
+#[derive(Debug, Clone)]
+pub struct ExternalDocumentObject {
+    url: String,
+    description: Option<String>,
+}
+
+impl<T: Into<String>> From<T> for ExternalDocumentObject {
+    fn from(url: T) -> Self {
+        Self::new(url)
+    }
+}
+
+impl ExternalDocumentObject {
+    /// Create a external document object by url.
+    pub fn new(url: impl Into<String>) -> ExternalDocumentObject {
+        Self {
+            url: url.into(),
+            description: None,
+        }
+    }
+
+    /// Sets a description of the target documentation..
+    pub fn description(self, description: impl Into<String>) -> Self {
+        Self {
+            description: Some(description.into()),
+            ..self
+        }
+    }
+}
+
 /// An OpenAPI service for Poem.
-pub struct OpenApiService<T> {
+pub struct OpenApiService<T, W: ?Sized> {
     api: T,
+    _webhook: PhantomData<W>,
     info: MetaInfo,
+    external_document: Option<MetaExternalDocument>,
     servers: Vec<MetaServer>,
     cookie_key: Option<CookieKey>,
 }
 
-impl<T> OpenApiService<T> {
+impl<T> OpenApiService<T, ()> {
     /// Create an OpenAPI container.
     #[must_use]
     pub fn new(api: T, title: impl Into<String>, version: impl Into<String>) -> Self {
         Self {
             api,
+            _webhook: PhantomData,
             info: MetaInfo {
                 title: title.into(),
                 description: None,
@@ -115,10 +139,25 @@ impl<T> OpenApiService<T> {
                 terms_of_service: None,
                 license: None,
             },
+            external_document: None,
             servers: Vec::new(),
             cookie_key: None,
         }
     }
+}
+
+impl<T, W: ?Sized> OpenApiService<T, W> {
+    /// Sets the webhooks.
+    pub fn webhooks<W2: ?Sized>(self) -> OpenApiService<T, W2> {
+        OpenApiService {
+            api: self.api,
+            _webhook: PhantomData,
+            info: self.info,
+            external_document: self.external_document,
+            servers: self.servers,
+            cookie_key: self.cookie_key,
+        }
+    }
 
     /// Sets the description of the API container.
     #[must_use]
@@ -146,7 +185,7 @@ impl<T> OpenApiService<T> {
         self
     }
 
-    /// Specifies the license information for the exposed API.
+    /// Sets the license information for the exposed API.
     ///
     /// Reference: <https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.1.0.md#license-object>
     #[must_use]
@@ -160,6 +199,22 @@ impl<T> OpenApiService<T> {
         self
     }
 
+    /// Add a external document object.
+    ///
+    /// Reference: <https://github.com/OAI/OpenAPI-Specification/blob/main/versions/3.1.0.md#external-documentation-object>
+    #[must_use]
+    pub fn external_document(
+        mut self,
+        external_document: impl Into<ExternalDocumentObject>,
+    ) -> Self {
+        let external_document = external_document.into();
+        self.external_document = Some(MetaExternalDocument {
+            url: external_document.url,
+            description: external_document.description,
+        });
+        self
+    }
+
     /// Sets the cookie key.
     pub fn cookie_key(self, key: CookieKey) -> Self {
         Self {
@@ -174,6 +229,7 @@ impl<T> OpenApiService<T> {
     pub fn swagger_ui(&self) -> impl Endpoint
     where
         T: OpenApi,
+        W: Webhook,
     {
         crate::ui::swagger_ui::create_endpoint(&self.spec())
     }
@@ -184,6 +240,7 @@ impl<T> OpenApiService<T> {
     pub fn rapidoc(&self) -> impl Endpoint
     where
         T: OpenApi,
+        W: Webhook,
     {
         crate::ui::rapidoc::create_endpoint(&self.spec())
     }
@@ -194,6 +251,7 @@ impl<T> OpenApiService<T> {
     pub fn redoc(&self) -> impl Endpoint
     where
         T: OpenApi,
+        W: Webhook,
     {
         crate::ui::redoc::create_endpoint(&self.spec())
     }
@@ -202,6 +260,7 @@ impl<T> OpenApiService<T> {
     pub fn spec_endpoint(&self) -> impl Endpoint
     where
         T: OpenApi,
+        W: Webhook,
     {
         let spec = self.spec();
         make_sync(move |_| {
@@ -215,22 +274,28 @@ impl<T> OpenApiService<T> {
     pub fn spec(&self) -> String
     where
         T: OpenApi,
+        W: Webhook,
     {
         let mut registry = Registry::new();
         let metadata = T::meta();
         T::register(&mut registry);
+        W::register(&mut registry);
+
+        let webhooks = W::meta();
 
         let doc = Document {
             info: &self.info,
             servers: &self.servers,
             apis: &metadata,
+            webhooks: &webhooks,
             registry: &registry,
+            external_document: self.external_document.as_ref(),
         };
         serde_json::to_string_pretty(&doc).unwrap()
     }
 }
 
-impl<T: OpenApi> IntoEndpoint for OpenApiService<T> {
+impl<T: OpenApi, W: Webhook> IntoEndpoint for OpenApiService<T, W> {
     type Endpoint = BoxEndpoint<'static, Response>;
 
     fn into_endpoint(self) -> Self::Endpoint {
@@ -245,6 +310,22 @@ impl<T: OpenApi> IntoEndpoint for OpenApiService<T> {
             None => CookieJarManager::new(),
         };
 
+        // check duplicate operation id
+        let mut operation_ids = HashSet::new();
+        for operation in T::meta()
+            .into_iter()
+            .map(|api| api.paths.into_iter())
+            .flatten()
+            .map(|path| path.operations.into_iter())
+            .flatten()
+        {
+            if let Some(operation_id) = operation.operation_id {
+                if !operation_ids.insert(operation_id) {
+                    panic!("duplicate operation id: {}", operation_id);
+                }
+            }
+        }
+
         self.api
             .add_routes(Route::new())
             .with(cookie_jar_manager)
